searchNodes=[{"doc":"IP lookup, with longest prefix match, for IPv4, IPv6 prefixes (and others). Iptrie manages multiple Radix trees, one for each type of prefix used as determined by their maxlen property after encoding it as a Pfx.t/0 struct. That way, IPv4 prefixes ( maxlen: 32 ) use a different radix tree as opposed to IPv6 ( maxlen: 128 ) or other types. Iptrie has a bias towards IPv4, IPv6, EUI-48 and EUI-64, since it uses Pfx to convert arguments to a Pfx.t/0 struct. Other types of prefixes will require the actual Pfx.t/0 structs as arguments for the various Iptrie functions. Like Pfx , Iptrie tries to mirror the representation of results to the arguments given, if possible. IPv4/IPv6 iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.2.3.0/24&quot; , &quot;v4&quot; ) ...&gt; |&gt; put ( &quot;128.0.0.0/8&quot; , &quot;v4-128&quot; ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; ) ...&gt; |&gt; put ( &quot;acdc:1978::/32&quot; , &quot;Powerage&quot; ) ...&gt; |&gt; put ( &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; ) ...&gt; |&gt; put ( &quot;::/0&quot; , &quot;no dynamite&quot; ) iex&gt; iex&gt; # 3 longest prefix matches that find the same prefix iex&gt; lookup ( ipt , &quot;1.2.3.128&quot; ) { &quot;1.2.3.0/24&quot; , &quot;v4&quot; } iex&gt; lookup ( ipt , { { 1 , 2 , 3 , 128 } , 32 } ) { { { 1 , 2 , 3 , 0 } , 24 } , &quot;v4&quot; } iex&gt; iex&gt; lookup ( ipt , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 128 &gt;&gt; , maxlen : 32 } ) { % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } , &quot;v4&quot; } iex&gt; iex&gt; lookup ( ipt , &quot;acdc:1975::&quot; ) { &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; } iex&gt; iex&gt; # separate trees, separate default &quot;routes&quot; iex&gt; lookup ( ipt , &quot;10.11.12.13&quot; ) { &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; } iex&gt; lookup ( ipt , &quot;abba::&quot; ) { &quot;::/0&quot; , &quot;no dynamite&quot; } iex&gt; iex&gt; # visualize the IPv4 &amp; IPv6 radix trees iex&gt; kv32 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 32 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 32 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv4&quot; , kv_tostr : kv32 ) ...&gt; |&gt; ( &amp; File . write ( &quot;assets/ipv4.dot&quot; , &amp;1 ) ) . ( ) iex&gt; kv128 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 128 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 128 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv6&quot; , kv_tostr : kv128 ) ...&gt; |&gt; ( &amp; File . write ( &quot;assets/ipv6.dot&quot; , &amp;1 ) ) . ( ) The radix trees in the example above look like this: Others Iptrie can also be used to do longest prefix match lookup for other types of prefixes, like e.g. MAC addresses: iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;00-22-72-00-00-00/24&quot; , &quot;American Micro-Fuel Device&quot; ) ...&gt; |&gt; put ( &quot;00-d0-ef-00-00-00/24&quot; , &quot;IGT&quot; ) ...&gt; |&gt; put ( &quot;08-61-95-00-00-00/24&quot; , &quot;Rockwell Automation&quot; ) iex&gt; iex&gt; lookup ( ipt , &quot;00-d0-ef-aa-bb-cc&quot; ) { &quot;00-D0-EF-00-00-00/24&quot; , &quot;IGT&quot; } iex&gt; iex&gt; # longest match for partial prefix iex&gt; lookup ( ipt , &quot;08-61-95-11-22-00/40&quot; ) |&gt; elem ( 1 ) &quot;Rockwell Automation&quot; iex&gt; iex&gt; kv48 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 48 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 48 ) ...&gt; |&gt; Radix . dot ( label : &quot;MAC OUI&quot; , kv_tostr : kv48 ) ...&gt; |&gt; ( &amp; File . write ( &quot;assets/mac.dot&quot; , &amp;1 ) ) . ( ) Iptrie recognizes EUI-48 and EUI-64 prefixes. Note that EUI-64 using ':' as punctuation might come out as an IPv6, in which case Pfx.from_mac/1 should be used to create a prefix before putting it in the trie. Since prefixes are stored in specific radix trees based on the maxlen of given prefix, you could also mix IPv4, IPv6, EUI-48, EUI-64 prefixes and possibly others, in a single Iptrie. IANA Special-Purpose Registries The Iptrie.Iana module enables retrieval of properties for prefixes registered in one of the IPv4 or IPv6 special-purpose address registries. iex&gt; alias Iptrie.Iana iex&gt; Iana . lookup ( &quot;10.11.12.13&quot; ) { &quot;10.0.0.0/8&quot; , %{ allocation : &quot;1996-02&quot; , destination : true , forward : true , global : false , name : &quot;private-use&quot; , prefix : &quot;10.0.0.0/8&quot; , reserved : false , source : true , spec : [ &quot;rfc1918&quot; ] , termination : :na } } # or access some property directly iex&gt; alias Iptrie.Iana iex&gt; Iana . lookup ( &quot;2001::&quot; , :name ) &quot;teredo&quot; Examples Minimize an Iptrie to its bare minimum by combining neighbouring prefixes and removing more specific prefixes. iex&gt; f = fn v1 , v2 -&gt; if v1 == v2 , do : { :ok , v1 } , else : nil end iex&gt; [ ...&gt; { &quot;1.1.1.0/25&quot; , true } , ...&gt; { &quot;1.1.1.128/25&quot; , true } , ...&gt; { &quot;2.2.2.0/24&quot; , true } , ...&gt; { &quot;2.2.2.0/28&quot; , true } , ...&gt; { &quot;2.2.2.2&quot; , true } ...&gt; ] ...&gt; |&gt; new ( ) ...&gt; |&gt; minimize ( f ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { k , v } -&gt; { &quot; \#{ k } &quot; , v } end ) [ { &quot;1.1.1.0/24&quot; , true } , { &quot;2.2.2.0/24&quot; , true } ] Summarize an Iptrie containing a list of full length prefixes within a /24, with 1 missing address: iex&gt; new ( for x &lt;- 0 .. 255 , do : { &quot;1.1.1. \#{ x } &quot; , x } ) ...&gt; |&gt; delete ( &quot;1.1.1.128&quot; ) ...&gt; |&gt; prune ( fn _ -&gt; { :ok , 0 } end , recurse : true ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { pfx , _v } -&gt; { &quot; \#{ pfx } &quot; , &quot;Range \#{ Pfx . first ( pfx ) } - \#{ Pfx . last ( pfx ) } &quot; } end ) [ { &quot;1.1.1.0/25&quot; , &quot;Range 1.1.1.0 - 1.1.1.127&quot; } , { &quot;1.1.1.129&quot; , &quot;Range 1.1.1.129 - 1.1.1.129&quot; } , { &quot;1.1.1.130/31&quot; , &quot;Range 1.1.1.130 - 1.1.1.131&quot; } , { &quot;1.1.1.132/30&quot; , &quot;Range 1.1.1.132 - 1.1.1.135&quot; } , { &quot;1.1.1.136/29&quot; , &quot;Range 1.1.1.136 - 1.1.1.143&quot; } , { &quot;1.1.1.144/28&quot; , &quot;Range 1.1.1.144 - 1.1.1.159&quot; } , { &quot;1.1.1.160/27&quot; , &quot;Range 1.1.1.160 - 1.1.1.191&quot; } , { &quot;1.1.1.192/26&quot; , &quot;Range 1.1.1.192 - 1.1.1.255&quot; } ] Find the more specific entries in an Iptrie for a given search prefix iex&gt; new ( ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;TNT&quot; ) ...&gt; |&gt; put ( &quot;acdc:1977::/32&quot; , &quot;LTBR&quot; ) ...&gt; |&gt; put ( &quot;abba:1975::/32&quot; , &quot;RING&quot; ) ...&gt; |&gt; more ( &quot;acdc::/16&quot; ) ...&gt; |&gt; Enum . map ( fn { p , v } -&gt; { &quot; \#{ p } &quot; , v } end ) [ { &quot;acdc:1977::/32&quot; , &quot;LTBR&quot; } , { &quot;acdc:1975::/32&quot; , &quot;TNT&quot; } ] Find the less specific entries in an Iptrie for a given search prefix iex&gt; new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 0 ) ...&gt; |&gt; put ( &quot;1.1.1.0/25&quot; , 1 ) ...&gt; |&gt; put ( &quot;1.1.1.0/26&quot; , 2 ) ...&gt; |&gt; put ( &quot;1.1.1.0/30&quot; , 3 ) ...&gt; |&gt; put ( &quot;1.1.1.128/25&quot; , 4 ) ...&gt; |&gt; less ( &quot;1.1.1.3&quot; ) ...&gt; |&gt; Enum . map ( fn { pfx , v } -&gt; { &quot; \#{ pfx } &quot; , v } end ) [ { &quot;1.1.1.0/30&quot; , 3 } , { &quot;1.1.1.0/26&quot; , 2 } , { &quot;1.1.1.0/25&quot; , 1 } , { &quot;1.1.1.0/24&quot; , 0 } ]","ref":"Iptrie.html","title":"Iptrie","type":"module"},{"doc":"Returns the number of prefix,value-pairs in given trie . Note that this requires traversal of radix tree(s) present in trie . Example iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } , { &quot;acdc::&quot; , 2 } ] ) iex&gt; count ( t ) 2","ref":"Iptrie.html#count/1","title":"Iptrie.count/1","type":"function"},{"doc":"Returns the number of prefix,value-pairs for given type in trie . If trie has no radix tree of given type , 0 is returned. Use Iptrie.has_type?/2 to check if a trie holds a given type. Example iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } , { &quot;acdc::&quot; , 2 } ] ) iex&gt; count ( t , 32 ) 1 iex&gt; count ( t , 128 ) 1 iex&gt; types ( t ) ...&gt; |&gt; Enum . map ( fn type -&gt; { type , count ( t , type ) } end ) [ { 32 , 1 } , { 128 , 1 } ]","ref":"Iptrie.html#count/2","title":"Iptrie.count/2","type":"function"},{"doc":"Deletes a prefix,value-pair from trie using an exact match for prefix . If the prefix does not exist in the trie , the latter is returned unchanged. Examples iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) iex&gt; iex&gt; for pfx &lt;- keys ( ipt ) , do : &quot; \#{ pfx } &quot; [ &quot;1.1.1.0/24&quot; , &quot;2.2.2.0/24&quot; ] iex&gt; iex&gt; ipt = delete ( ipt , &quot;1.1.1.0/24&quot; ) iex&gt; for pfx &lt;- keys ( ipt ) , do : &quot; \#{ pfx } &quot; [ &quot;2.2.2.0/24&quot; ]","ref":"Iptrie.html#delete/2","title":"Iptrie.delete/2","type":"function"},{"doc":"Drops given prefixes from trie using an exact match. If a given prefix does not exist in trie it is ignored. Example # drop 2 existing prefixes and ignore the third iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;11-22-33-00-00-00/24&quot; , 3 } ] ) iex&gt; t2 = drop ( t , [ &quot;1.1.1.0/24&quot; , &quot;11-22-33-00-00-00/24&quot; , &quot;3.3.3.3&quot; ] ) iex&gt; for pfx &lt;- keys ( t2 ) , do : &quot; \#{ pfx } &quot; [ &quot;2.2.2.0/24&quot; ]","ref":"Iptrie.html#drop/2","title":"Iptrie.drop/2","type":"function"},{"doc":"Returns true if the given trie is empty, false otherwise. Examples iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } , { &quot;11-22-33-44-55-66&quot; , 2 } ] ) iex&gt; empty? ( t ) false iex&gt; new ( ) |&gt; empty? ( ) true","ref":"Iptrie.html#empty?/1","title":"Iptrie.empty?/1","type":"function"},{"doc":"Returns true if the radix tree for given type in trie is empty, false otherwise. Example iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } , { &quot;11-22-33-44-55-66&quot; , 2 } ] ) iex&gt; empty? ( t , 32 ) false iex&gt; empty? ( t , 48 ) false iex&gt; empty? ( t , 128 ) true","ref":"Iptrie.html#empty?/2","title":"Iptrie.empty?/2","type":"function"},{"doc":"Fetches the prefix,value-pair for given prefix from trie . Returns one of: {:ok, {prefix, value}} in case of success {:error, :notfound} if prefix is not present in trie {:error, :einval} in case of an invalid prefix , and {:error, :bad_trie} in case trie is not an Iptrie.t/0 Optionally fetches based on a longest prefix match by specifying match: :lpm . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) iex&gt; iex&gt; fetch ( ipt , &quot;1.1.1.0/24&quot; ) { :ok , { &quot;1.1.1.0/24&quot; , &quot;one&quot; } } iex&gt; iex&gt; fetch ( ipt , &quot;1.1.1.1&quot; ) { :error , :notfound } iex&gt; iex&gt; fetch ( ipt , &quot;1.1.1.1&quot; , match : :lpm ) { :ok , { &quot;1.1.1.0/24&quot; , &quot;one&quot; } } iex&gt; iex&gt; fetch ( ipt , &quot;13.13.13.333&quot; ) { :error , :einval }","ref":"Iptrie.html#fetch/3","title":"Iptrie.fetch/3","type":"function"},{"doc":"Fetches the prefix,value-pair for given prefix from trie . In case of success, returns {prefix, value} . If prefix is not present, raises a KeyError . If prefix could not be encoded, raises an ArgumentError . Optionally fetches based on a longest prefix match by specifying match: :lpm . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;10.10.10.0/24&quot; , &quot;ten&quot; ) ...&gt; |&gt; put ( &quot;11.11.11.0/24&quot; , &quot;eleven&quot; ) iex&gt; iex&gt; fetch! ( ipt , &quot;10.10.10.0/24&quot; ) { &quot;10.10.10.0/24&quot; , &quot;ten&quot; } iex&gt; iex&gt; fetch! ( ipt , &quot;11.11.11.11&quot; , match : :lpm ) { &quot;11.11.11.0/24&quot; , &quot;eleven&quot; } iex&gt; iex&gt; fetch! ( ipt , &quot;12.12.12.12&quot; ) ** (KeyError) prefix &quot;12.12.12.12&quot; not found iex&gt; ipt = new ( ) iex&gt; fetch! ( ipt , &quot;13.13.13.333&quot; ) ** (ArgumentError) expected a valid prefix, got &quot;13.13.13.333&quot;","ref":"Iptrie.html#fetch!/3","title":"Iptrie.fetch!/3","type":"function"},{"doc":"Returns a new Iptrie, keeping only the prefix,value-pairs for which fun returns truthy . The signature for fun is (prefix, value -&gt; boolean), where the value is stored under prefix in the trie. Radix trees that are empty, are removed from the new Iptrie. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;rock&quot; ) ...&gt; |&gt; put ( &quot;acdc:1976::/32&quot; , &quot;rock&quot; ) ...&gt; |&gt; put ( &quot;abba:1975::/32&quot; , &quot;pop&quot; ) ...&gt; |&gt; put ( &quot;abba:1976::/32&quot; , &quot;pop&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;v4&quot; ) iex&gt; iex&gt; filter ( ipt , fn pfx , _value -&gt; pfx . maxlen == 32 end ) ...&gt; |&gt; to_list ( ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , &quot;v4&quot; } ] iex&gt; iex&gt; filter ( ipt , fn _pfx , value -&gt; value == &quot;rock&quot; end ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { pfx , value } -&gt; { &quot; \#{ pfx } &quot; , value } end ) [ { &quot;acdc:1975::/32&quot; , &quot;rock&quot; } , { &quot;acdc:1976::/32&quot; , &quot;rock&quot; } ]","ref":"Iptrie.html#filter/2","title":"Iptrie.filter/2","type":"function"},{"doc":"Finds a prefix,value-pair for given prefix from trie using a longest prefix match. Convenience wrapper for Iptrie.fetch/3 with match: :lpm . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;one&quot; ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , &quot;two&quot; ) iex&gt; iex&gt; find ( ipt , &quot;1.1.1.0/24&quot; ) { :ok , { &quot;1.1.1.0/24&quot; , &quot;one&quot; } } iex&gt; iex&gt; find ( ipt , &quot;12.12.12.12&quot; ) { :error , :notfound } iex&gt; iex&gt; find ( ipt , &quot;13.13.13.333&quot; ) { :error , :einval }","ref":"Iptrie.html#find/2","title":"Iptrie.find/2","type":"function"},{"doc":"Finds a prefix,value-pair for given prefix from trie using a longest prefix match. Convenience wrapper for Iptrie.fetch!/3 with match: :lpm . Examples iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;10.10.10.0/24&quot; , &quot;ten&quot; ) ...&gt; |&gt; put ( &quot;11.11.11.0/24&quot; , &quot;eleven&quot; ) iex&gt; iex&gt; find! ( ipt , &quot;10.10.10.0/24&quot; ) { &quot;10.10.10.0/24&quot; , &quot;ten&quot; } iex&gt; iex&gt; find! ( ipt , &quot;10.10.10.10&quot; ) { &quot;10.10.10.0/24&quot; , &quot;ten&quot; } iex&gt; iex&gt; find! ( ipt , &quot;12.12.12.12&quot; ) ** (KeyError) prefix &quot;12.12.12.12&quot; not found iex&gt; ipt = new ( ) iex&gt; find! ( ipt , &quot;13.13.13.333&quot; ) ** (ArgumentError) expected a valid prefix, got &quot;13.13.13.333&quot;","ref":"Iptrie.html#find!/2","title":"Iptrie.find!/2","type":"function"},{"doc":"Returns the prefix,value-pair stored under given prefix in trie , using an exact match. If prefix is not found, default is returned. If default is not provided, nil is used. Example iex&gt; ipt = new ( [ { &quot;1.1.1.0/30&quot; , &quot;A&quot; } , { &quot;1.1.1.0/31&quot; , &quot;B&quot; } , { &quot;1.1.1.0&quot; , &quot;C&quot; } ] ) iex&gt; iex&gt; get ( ipt , &quot;1.1.1.0/31&quot; ) { &quot;1.1.1.0/31&quot; , &quot;B&quot; } iex&gt; iex&gt; get ( ipt , &quot;2.2.2.0/30&quot; ) nil iex&gt; get ( ipt , &quot;2.2.2.0/30&quot; , :notfound ) :notfound","ref":"Iptrie.html#get/3","title":"Iptrie.get/3","type":"function"},{"doc":"Updates a key,value-pair in trie by invoking fun with the result of an exact match. The callback fun is called with: {bits, original_value} if an exact match was found, or nil , in case the search prefix is not present in the trie where bits are the actual bits of given prefix as retrieved from the corresponding radix tree in given trie . The callback function should return: {current_value, new_value} , or :pop . When {current_value, new_value} is returned: the new_value is stored in the trie under given prefix , and {current_value, trie} is returned. When the callback passes back :pop : the {prefix, original_value} -pair is deleted from the trie , and {original_value, trie} is returned. If the callback passes back :pop when its argument was nil then {nil, trie} is returned, where trie is unchanged. If something similar is required, but based on a longest prefix match, perhaps Iptrie.update/3 or Iptrie.update/4 is better suited. Example iex&gt; counter = fn { _ , v } -&gt; { v , v + 1 } ...&gt; nil -&gt; { 0 , 1 } ...&gt; end iex&gt; ipt = new ( ) iex&gt; { org , ipt } = get_and_update ( ipt , &quot;1.1.1.1&quot; , counter ) iex&gt; org 0 iex&gt; get ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.1&quot; , 1 } iex&gt; { org , ipt } = get_and_update ( ipt , &quot;1.1.1.1/24&quot; , counter ) iex&gt; org 0 iex&gt; get ( ipt , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 } iex&gt; { org , ipt } = get_and_update ( ipt , &quot;1.1.1.0/24&quot; , fn _ -&gt; :pop end ) iex&gt; org 1 iex&gt; get ( ipt , &quot;1.1.1.0/24&quot; ) nil # aggregate stats on a /24 iex&gt; count = fn { _ , v } -&gt; { v , v + 1 } ...&gt; nil -&gt; { 0 , 1 } ...&gt; end iex&gt; track = fn ip -&gt; Pfx . new ( ip ) |&gt; Pfx . keep ( 24 ) end iex&gt; iex&gt; ipt = new ( ) iex&gt; { _ , ipt } = get_and_update ( ipt , track . ( &quot;1.1.1.1&quot; ) , count ) iex&gt; { _ , ipt } = get_and_update ( ipt , track . ( { 1 , 1 , 1 , 2 } ) , count ) iex&gt; { _ , ipt } = get_and_update ( ipt , track . ( &quot;acdc::/64&quot; ) , count ) iex&gt; { org , ipt } = get_and_update ( ipt , track . ( { { 1 , 1 , 1 , 3 } , 32 } ) , count ) iex&gt; org 2 iex&gt; get ( ipt , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 3 } iex&gt; get ( ipt , &quot;acdc::/24&quot; ) { &quot;acdc::/24&quot; , 1 } # note that Pfx.keep({1, 1, 1, 2}, 24) yields {1, 1, 1, 0} since its # return value mimics its argument and thus results in a full prefix, # hence the need for track.(ip)","ref":"Iptrie.html#get_and_update/3","title":"Iptrie.get_and_update/3","type":"function"},{"doc":"Returns true if given prefix is present in trie , false otherwise. The check is done based on an exact match, unless the option match: :lpm is provided to match based on longest prefix match. Example iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } , { &quot;1.1.1.0/24&quot; , 2 } , { &quot;acdc::/16&quot; , 3 } ] ) iex&gt; has_prefix? ( t , &quot;1.1.1.2&quot; ) false iex&gt; has_prefix? ( t , &quot;1.1.1.2&quot; , match : :lpm ) true iex&gt; has_prefix? ( t , &quot;1.1.1.1&quot; ) true iex&gt; has_prefix? ( t , &quot;acdc::/16&quot; ) true","ref":"Iptrie.html#has_prefix?/3","title":"Iptrie.has_prefix?/3","type":"function"},{"doc":"Returns true if trie has given type , false otherwise. An Iptrie groups prefixes into radix trees by their maxlen property, also known as the type of prefix. Use Iptrie.types/1 to get a list of all available types. Example iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } ] ) iex&gt; has_type? ( t , 32 ) true iex&gt; has_type? ( t , 128 ) false","ref":"Iptrie.html#has_type?/2","title":"Iptrie.has_type?/2","type":"function"},{"doc":"See Iptrie.Iana.lookup/2 .","ref":"Iptrie.html#iana_special/2","title":"Iptrie.iana_special/2","type":"function"},{"doc":"Returns all prefixes stored in all available radix trees in trie . The prefixes are reconstructed as Pfx.t/0 by combining the stored bitstrings with the Radix -tree's type, that is the maxlen property associated with the radix tree whose keys are being retrieved. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; keys ( ipt ) [ % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } ]","ref":"Iptrie.html#keys/1","title":"Iptrie.keys/1","type":"function"},{"doc":"Returns the prefixes stored in the radix tree in trie for given type . Note that the Iptrie keys are returned as Pfx.t/0 structs. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; keys ( ipt , 32 ) [ % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } ] iex&gt; iex&gt; keys ( ipt , 128 ) [ % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } ] iex&gt; iex&gt; keys ( ipt , 48 ) [ ]","ref":"Iptrie.html#keys/2","title":"Iptrie.keys/2","type":"function"},{"doc":"Returns all the prefix,value-pairs whose prefix is a prefix for the given search prefix . This returns the less specific entries that enclose the given search prefix . Note that any bitstring is always a prefix of itself. So, unless the option :exclude is set to true , the search key will be included in the result if present. Any other value for :exclude is interpreted as false, which means the default action is to include the search key, if present. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/30&quot; , &quot;A30&quot; ) ...&gt; |&gt; put ( &quot;1.1.2.0/24&quot; , &quot;B24&quot; ) iex&gt; iex&gt; less ( ipt , &quot;1.1.1.0/30&quot; ) [ { &quot;1.1.1.0/30&quot; , &quot;A30&quot; } , { &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; } , ] iex&gt; less ( ipt , &quot;2.2.2.2&quot; ) [ ] # # exclusive search # iex&gt; less ( ipt , &quot;1.1.1.0/30&quot; , exclude : true ) [ { &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; } ]","ref":"Iptrie.html#less/3","title":"Iptrie.less/3","type":"function"},{"doc":"Returns the prefix,value-pair, whose prefix is the longest match for given search prefix . Returns nil if there is no match for search prefix . Examples iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; lookup ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.0/24&quot; , 1 } iex&gt; lookup ( ipt , &quot;acdc:1975:1::&quot; ) { &quot;acdc:1975::/32&quot; , 3 } iex&gt; iex&gt; lookup ( ipt , &quot;3.3.3.3&quot; ) nil iex&gt; lookup ( ipt , &quot;3.3.3.300&quot; ) ** (ArgumentError) expected an ipv4/ipv6 CIDR or EUI-48/64 string, got &quot;3.3.3.300&quot;","ref":"Iptrie.html#lookup/2","title":"Iptrie.lookup/2","type":"function"},{"doc":"Merges trie1 and trie2 into a new Iptrie. Adds all prefix,value-pairs of trie2 to trie1 , overwriting any existing entries when prefixes match (based on exact match). Example iex&gt; t1 = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } ] ) iex&gt; t2 = new ( [ { &quot;2.2.2.0/24&quot; , 22 } , { &quot;3.3.3.0/24&quot; , 3 } ] ) iex&gt; t = merge ( t1 , t2 ) iex&gt; count ( t ) 3 iex&gt; get ( t , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 } iex&gt; get ( t , &quot;2.2.2.0/24&quot; ) { &quot;2.2.2.0/24&quot; , 22 } iex&gt; get ( t , &quot;3.3.3.0/24&quot; ) { &quot;3.3.3.0/24&quot; , 3 }","ref":"Iptrie.html#merge/2","title":"Iptrie.merge/2","type":"function"},{"doc":"Merges trie1 and trie2 into a new Iptrie, resolving conflicts through fun . In cases where a prefix is present in both tries, the conflict is resolved by calling fun with the prefix (a Pfx.t/0 ), its value in trie1 and its value in trie2 . The function's return value will be stored under the prefix in the merged trie. Example iex&gt; t1 = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;acdc:1975::/32&quot; , 3 } ] ) iex&gt; t2 = new ( [ { &quot;3.3.3.0/24&quot; , 4 } , { &quot;2.2.2.0/24&quot; , 5 } , { &quot;acdc:2021::/32&quot; , 6 } ] ) iex&gt; t = merge ( t1 , t2 , fn _pfx , v1 , v2 -&gt; v1 + v2 end ) iex&gt; count ( t ) 5 iex&gt; get ( t , &quot;2.2.2.0/24&quot; ) { &quot;2.2.2.0/24&quot; , 7 } iex&gt; for ip4 &lt;- keys ( t , 32 ) , do : &quot; \#{ ip4 } &quot; [ &quot;1.1.1.0/24&quot; , &quot;2.2.2.0/24&quot; , &quot;3.3.3.0/24&quot; ] iex&gt; for ip6 &lt;- keys ( t , 128 ) , do : &quot; \#{ ip6 } &quot; [ &quot;acdc:1975::/32&quot; , &quot;acdc:2021::/32&quot; ] iex&gt; values ( t ) |&gt; Enum . sum ( ) 1 + 7 + 3 + 4 + 6","ref":"Iptrie.html#merge/3","title":"Iptrie.merge/3","type":"function"},{"doc":"Minimizes a trie by pruning and removing more specific prefixes, where possible. fun has function signature (any, any) -&gt; {:ok, any} | nil . Whenever prefixes are found that could be combined, fun is called with their values. If fun returns nil, nothing happens. In case of two neighbours, a non-existing parent and an {:ok, value} , the value is stored under the parent and the two neighbors are deleted. fun is called with the leftmost neighbour's value as its first argument while the second argument is the rightmost neighbour's value. In case of a less and a more specific prefix and an {:ok, value} , the value is stored under the less specific prefix and the more specific prefix is deleted. fun is called with the less spefic prefix' value as its first argument and the more specific prefix's value as its second argument. Examples Both neighbouring /25 s are combined (no parent) and the /26 is removed as well. iex&gt; f = fn v1 , v2 -&gt; if v1 == v2 , do : { :ok , v1 } , else : nil end iex&gt; [ { &quot;1.1.1.0/26&quot; , true } , { &quot;1.1.1.0/25&quot; , true } , { &quot;1.1.1.128/25&quot; , true } ] ...&gt; |&gt; new ( ) ...&gt; |&gt; minimize ( f ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { k , v } -&gt; { &quot; \#{ k } &quot; , v } end ) [ { &quot;1.1.1.0/24&quot; , true } ] The /25 s cannot be combined since their parent exists and has a different value. iex&gt; f = fn v1 , v2 -&gt; if v1 == v2 , do : { :ok , true } , else : nil end iex&gt; [ { &quot;1.1.1.0/24&quot; , false } , { &quot;1.1.1.0/25&quot; , true } , { &quot;1.1.1.128/25&quot; , true } ] ...&gt; |&gt; new ( ) ...&gt; |&gt; minimize ( f ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { k , v } -&gt; { &quot; \#{ k } &quot; , v } end ) [ { &quot;1.1.1.0/25&quot; , true } , { &quot;1.1.1.0/24&quot; , false } , { &quot;1.1.1.128/25&quot; , true } ] Only 1.1.1.128/25 can be combined with 1.1.1.0/24 . iex&gt; f = fn v1 , v2 -&gt; if v1 == v2 , do : { :ok , true } , else : nil end iex&gt; [ { &quot;1.1.1.0/24&quot; , true } , { &quot;1.1.1.0/25&quot; , false } , { &quot;1.1.1.128/25&quot; , true } ] ...&gt; |&gt; new ( ) ...&gt; |&gt; minimize ( f ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { k , v } -&gt; { &quot; \#{ k } &quot; , v } end ) [ { &quot;1.1.1.0/25&quot; , false } , { &quot;1.1.1.0/24&quot; , true } ]","ref":"Iptrie.html#minimize/2","title":"Iptrie.minimize/2","type":"function"},{"doc":"Returns all the prefix,value-pairs where the search prefix is a prefix for the stored prefix. This returns the more specific entries that are enclosed by given search prefix . Note that any bitstring is always a prefix of itself. So, unless the option :exclude is set to true the search key, if present, will be included in the results. Any other value for :exclude is ignored, which means the default action is to include the search key (if present). Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/30&quot; , &quot;A30&quot; ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , &quot;A24&quot; ) iex&gt; iex&gt; more ( ipt , &quot;1.1.1.0/24&quot; ) [ { &quot;1.1.1.0/30&quot; , &quot;A30&quot; } , { &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; } , { &quot;1.1.1.0/24&quot; , &quot;A24&quot; } , { &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; } ] # # exclusive search # iex&gt; more ( ipt , &quot;1.1.1.0/24&quot; , exclude : true ) [ { &quot;1.1.1.0/30&quot; , &quot;A30&quot; } , { &quot;1.1.1.0/25&quot; , &quot;A25-lower&quot; } , { &quot;1.1.1.128/25&quot; , &quot;A25-upper&quot; } ]","ref":"Iptrie.html#more/3","title":"Iptrie.more/3","type":"function"},{"doc":"Creates an new, empty Iptrie. Example iex&gt; Iptrie . new ( ) % Iptrie { }","ref":"Iptrie.html#new/0","title":"Iptrie.new/0","type":"function"},{"doc":"Creates a new Iptrie, populated via a list of prefix,value-pairs. Example iex&gt; elements = [ ...&gt; { &quot;1.1.1.0/24&quot; , &quot;net1&quot; } , ...&gt; { { { 1 , 1 , 2 , 0 } , 24 } , &quot;net2&quot; } , ...&gt; { &quot;acdc:1975::/32&quot; , &quot;TNT&quot; } ...&gt; ] iex&gt; ipt = Iptrie . new ( elements ) iex&gt; radix ( ipt , 32 ) { 0 , { 22 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;net1&quot; } ] , [ { &lt;&lt; 1 , 1 , 2 &gt;&gt; , &quot;net2&quot; } ] } , nil } iex&gt; radix ( ipt , 128 ) { 0 , nil , [ { &lt;&lt; 172 , 220 , 25 , 117 &gt;&gt; , &quot;TNT&quot; } ] }","ref":"Iptrie.html#new/1","title":"Iptrie.new/1","type":"function"},{"doc":"Removes the value associated with prefix and returns the matched prefix,value-pair and the new Iptrie. Options include: default: value to return if prefix could not be matched (defaults to nil ) match: :lpm to match on longest prefix instead of an exact match Examples iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;1.1.1.99&quot; , 2 } , { &quot;acdc:1975::/32&quot; , 3 } ] ) iex&gt; { { &quot;1.1.1.99&quot; , 2 } , t2 } = pop ( t , &quot;1.1.1.99&quot; ) iex&gt; get ( t2 , &quot;1.1.1.99&quot; ) nil iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;1.1.1.99&quot; , 2 } , { &quot;acdc:1975::/32&quot; , 3 } ] ) iex&gt; # t is unchanged iex&gt; { { &quot;1.1.1.33&quot; , :notfound } , ^ t } = pop ( t , &quot;1.1.1.33&quot; , default : :notfound ) iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;1.1.1.99&quot; , 2 } , { &quot;acdc:1975::/32&quot; , 3 } ] ) iex&gt; # lpm match iex&gt; { { &quot;1.1.1.0/24&quot; , 1 } , t2 } = pop ( t , &quot;1.1.1.33&quot; , match : :lpm ) iex&gt; get ( t2 , &quot;1.1.1.0/24&quot; ) nil","ref":"Iptrie.html#pop/3","title":"Iptrie.pop/3","type":"function"},{"doc":"Prunes given trie by calling fun on neighboring prefixes, possibly replacing them with their parent. The callback fun is invoked with either a 5- or a 6-tuple: {p0, p1, v1, p2, v2} for neighboring p1 , p2 , their parent p0 is not in trie {p0, v0, p1, v1, p2, v2} for the parent, its value and that of its two neighboring children. The callback decides what happens by returning either: {:ok, value} , value will be stored under p0 and the neighboring prefixes p1, p2 are deleted nil (or anything else really), in which case the tree is not changed. Examples iex&gt; adder = fn ...&gt; { _p0 , _p1 , v1 , _p2 , v2 } -&gt; { :ok , v1 + v2 } ...&gt; { _p0 , v0 , _p1 , v1 , _p2 , v2 } -&gt; { :ok , v0 + v1 + v2 } ...&gt; end iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/26&quot; , 0 ) ...&gt; |&gt; put ( &quot;1.1.1.64/26&quot; , 1 ) ...&gt; |&gt; put ( &quot;1.1.1.128/26&quot; , 2 ) ...&gt; |&gt; put ( &quot;1.1.1.192/26&quot; , 3 ) iex&gt; prune ( ipt , adder ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { p , v } -&gt; { &quot; \#{ p } &quot; , v } end ) [ { &quot;1.1.1.0/25&quot; , 1 } , { &quot;1.1.1.128/25&quot; , 5 } ] iex&gt; iex&gt; prune ( ipt , adder , recurse : true ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { p , v } -&gt; { &quot; \#{ p } &quot; , v } end ) [ { &quot;1.1.1.0/24&quot; , 6 } ] # summerize all /24&#39;s inside 10.10.0.0/16 # -&gt; only 10.10.40.0/24 is missing iex&gt; slash24s = Pfx . partition ( &quot;10.10.0.0/16&quot; , 24 ) ...&gt; |&gt; Enum . with_index ( ) ...&gt; |&gt; new ( ) ...&gt; |&gt; delete ( &quot;10.10.40.0/24&quot; ) iex&gt; iex&gt; prune ( slash24s , fn _ -&gt; { :ok , 0 } end , recurse : true ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { p , v } -&gt; { &quot; \#{ p } &quot; , v } end ) [ { &quot;10.10.0.0/19&quot; , 0 } , { &quot;10.10.32.0/21&quot; , 0 } , { &quot;10.10.41.0/24&quot; , 41 } , { &quot;10.10.42.0/23&quot; , 0 } , { &quot;10.10.44.0/22&quot; , 0 } , { &quot;10.10.48.0/20&quot; , 0 } , { &quot;10.10.64.0/18&quot; , 0 } , { &quot;10.10.128.0/17&quot; , 0 } ]","ref":"Iptrie.html#prune/3","title":"Iptrie.prune/3","type":"function"},{"doc":"Puts the prefix,value-pairs in elements into trie . This always uses an exact match for prefix, updating its value if it exists. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( [ { &quot;1.1.1.0/24&quot; , 0 } , { &quot;1.1.1.1&quot; , 0 } , { &quot;1.1.1.1&quot; , &quot;x&quot; } ] ) iex&gt; iex&gt; get ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.1&quot; , &quot;x&quot; }","ref":"Iptrie.html#put/2","title":"Iptrie.put/2","type":"function"},{"doc":"Puts value under prefix in trie . This always uses an exact match for prefix , replacing its value if it exists. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 0 ) ...&gt; |&gt; put ( &quot;1.1.1.1&quot; , 1 ) ...&gt; |&gt; put ( &quot;1.1.1.1&quot; , &quot;x&quot; ) iex&gt; iex&gt; get ( ipt , &quot;1.1.1.1&quot; ) { &quot;1.1.1.1&quot; , &quot;x&quot; }","ref":"Iptrie.html#put/3","title":"Iptrie.put/3","type":"function"},{"doc":"Returns the Radix tree for given type in trie . If trie has no radix tree for given type it will return a new empty radix tree. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; radix ( ipt , 32 ) { 0 , { 6 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 1 } ] , [ { &lt;&lt; 2 , 2 , 2 &gt;&gt; , 2 } ] } , nil } iex&gt; iex&gt; radix ( ipt , 128 ) { 0 , nil , { 18 , [ { &lt;&lt; 172 , 220 , 25 , 117 &gt;&gt; , 3 } ] , [ { &lt;&lt; 172 , 220 , 32 , 33 &gt;&gt; , 4 } ] } } iex&gt; radix ( ipt , 48 ) { 0 , nil , nil } iex&gt; iex&gt; has_type? ( ipt , 48 ) false","ref":"Iptrie.html#radix/2","title":"Iptrie.radix/2","type":"function"},{"doc":"Invokes fun on all prefix,value-pairs in all radix trees in trie . The function fun is called with the prefix (a Pfx.t/0 struct), value and acc accumulator and should return an updated accumulator. The result is the last accumulator returned. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; reduce ( ipt , 0 , fn _pfx , value , acc -&gt; acc + value end ) 10 iex&gt; iex&gt; reduce ( ipt , %{ } , fn pfx , value , acc -&gt; Map . put ( acc , &quot; \#{ pfx } &quot; , value ) end ) %{ &quot;1.1.1.0/24&quot; =&gt; 1 , &quot;2.2.2.0/24&quot; =&gt; 2 , &quot;acdc:1975::/32&quot; =&gt; 3 , &quot;acdc:2021::/32&quot; =&gt; 4 }","ref":"Iptrie.html#reduce/3","title":"Iptrie.reduce/3","type":"function"},{"doc":"Invokes fun on each prefix,value-pair in the radix tree of given type in trie . The function fun is called with the prefix (a Pfx.t/0 struct), value and acc accumulator and should return an updated accumulator. The result is the last accumulator returned. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; reduce ( ipt , 32 , 0 , fn _pfx , value , acc -&gt; acc + value end ) 3 iex&gt; reduce ( ipt , 48 , 0 , fn _pfx , value , acc -&gt; acc + value end ) 0 iex&gt; reduce ( ipt , 128 , 0 , fn _pfx , value , acc -&gt; acc + value end ) 7","ref":"Iptrie.html#reduce/4","title":"Iptrie.reduce/4","type":"function"},{"doc":"Splits trie into two Iptries using given list of prefixes . Returns a new trie with prefix,value-pairs that were matched by given prefixes and the old trie with those pairs removed. If a prefix was not found in given trie it is ignored. By default an exact match is used, specify match: :lpm to use longest prefix match instead. Examples iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;3.3.3.0/30&quot; , 3 } ] ) iex&gt; { t2 , t3 } = split ( t , [ &quot;2.2.2.0/24&quot; , &quot;3.3.3.0/30&quot; ] ) iex&gt; count ( t2 ) 2 iex&gt; get ( t2 , &quot;2.2.2.0/24&quot; ) { &quot;2.2.2.0/24&quot; , 2 } iex&gt; get ( t2 , &quot;3.3.3.0/30&quot; ) { &quot;3.3.3.0/30&quot; , 3 } iex&gt; count ( t3 ) 1 iex&gt; get ( t3 , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 } # use longest prefix match iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;3.3.3.0/30&quot; , 3 } ] ) iex&gt; { t4 , t5 } = split ( t , [ &quot;2.2.2.2&quot; , &quot;3.3.3.3&quot; ] , match : :lpm ) iex&gt; count ( t4 ) 2 iex&gt; get ( t4 , &quot;2.2.2.0/24&quot; ) { &quot;2.2.2.0/24&quot; , 2 } iex&gt; get ( t4 , &quot;3.3.3.0/30&quot; ) { &quot;3.3.3.0/30&quot; , 3 } iex&gt; count ( t5 ) 1 iex&gt; get ( t5 , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 }","ref":"Iptrie.html#split/3","title":"Iptrie.split/3","type":"function"},{"doc":"Returns a new Iptrie containing only given prefixes that were found in trie . If a given prefix does not exist, it is ignored. Optionally specify match: :lpm to use a longest prefix match instead of exact, which is the default. Examples iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;acdc::/16&quot; , 3 } ] ) iex&gt; t2 = take ( t , [ &quot;1.1.1.0/24&quot; , &quot;acdc::/16&quot; ] ) iex&gt; count ( t2 ) 2 iex&gt; get ( t2 , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 } iex&gt; get ( t2 , &quot;acdc::/16&quot; ) { &quot;acdc::/16&quot; , 3 } # use longest match iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;acdc::/16&quot; , 3 } ] ) iex&gt; t3 = take ( t , [ &quot;1.1.1.1&quot; , &quot;acdc:1975::1&quot; ] , match : :lpm ) iex&gt; count ( t3 ) 2 iex&gt; get ( t3 , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 } iex&gt; get ( t3 , &quot;acdc::/16&quot; ) { &quot;acdc::/16&quot; , 3 } # ignore missing prefixes iex&gt; t = new ( [ { &quot;1.1.1.0/24&quot; , 1 } , { &quot;2.2.2.0/24&quot; , 2 } , { &quot;acdc::/16&quot; , 3 } ] ) iex&gt; t4 = take ( t , [ &quot;1.1.1.1&quot; , &quot;3.3.3.3&quot; ] , match : :lpm ) iex&gt; count ( t4 ) 1 iex&gt; get ( t4 , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 1 }","ref":"Iptrie.html#take/3","title":"Iptrie.take/3","type":"function"},{"doc":"Returns all prefix,value-pairs from all available radix trees in trie . Examples iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; to_list ( ipt ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , 1 } , { % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , 2 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , 3 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } , 4 } ]","ref":"Iptrie.html#to_list/1","title":"Iptrie.to_list/1","type":"function"},{"doc":"Returns the prefix,value-pairs from the radix trees in trie for given type . If the radix tree for type does not exist, an empty list is returned. Examples iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; to_list ( ipt , 32 ) [ { % Pfx { bits : &lt;&lt; 1 , 1 , 1 &gt;&gt; , maxlen : 32 } , 1 } , { % Pfx { bits : &lt;&lt; 2 , 2 , 2 &gt;&gt; , maxlen : 32 } , 2 } ] iex&gt; to_list ( ipt , 128 ) [ { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x1975 :: 16 &gt;&gt; , maxlen : 128 } , 3 } , { % Pfx { bits : &lt;&lt; 0xacdc :: 16 , 0x2021 :: 16 &gt;&gt; , maxlen : 128 } , 4 } ] iex&gt; to_list ( ipt , 48 ) [ ]","ref":"Iptrie.html#to_list/2","title":"Iptrie.to_list/2","type":"function"},{"doc":"Returns a list of types available in given trie . Example iex&gt; t = new ( [ { &quot;1.1.1.1&quot; , 1 } , { &quot;2001:db8::&quot; , 2 } ] ) iex&gt; types ( t ) [ 32 , 128 ]","ref":"Iptrie.html#types/1","title":"Iptrie.types/1","type":"function"},{"doc":"Looks up prefix and update the matched entry, only if found. Uses longest prefix match, so search prefix is usually matched by some less specific prefix. If matched, fun is called on its value. If prefix had no longest prefix match, the trie is returned unchanged. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 0 ) ...&gt; |&gt; update ( &quot;1.1.1.0&quot; , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;1.1.1.1&quot; , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;2.2.2.2&quot; , fn x -&gt; x + 1 end ) iex&gt; get ( ipt , &quot;1.1.1.0/24&quot; ) { &quot;1.1.1.0/24&quot; , 2 } iex&gt; lookup ( ipt , &quot;2.2.2.2&quot; ) nil","ref":"Iptrie.html#update/3","title":"Iptrie.update/3","type":"function"},{"doc":"Looks up prefix and, if found, updates its value or insert the default under prefix . Uses longest prefix match, so search prefix is usually matched by some less specific prefix. If matched, fun is called on the entry's value. If prefix had no longest prefix match, the default is inserted under prefix and fun is not called. Example iex&gt; ipt = new ( ) ...&gt; |&gt; update ( &quot;1.1.1.0/24&quot; , 0 , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;1.1.1.0&quot; , 0 , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;1.1.1.1&quot; , 0 , fn x -&gt; x + 1 end ) ...&gt; |&gt; update ( &quot;2.2.2.2&quot; , 0 , fn x -&gt; x + 1 end ) iex&gt; lookup ( ipt , &quot;1.1.1.2&quot; ) { &quot;1.1.1.0/24&quot; , 2 } iex&gt; iex&gt; # probably not what you wanted: iex&gt; lookup ( ipt , &quot;2.2.2.2&quot; ) { &quot;2.2.2.2&quot; , 0 }","ref":"Iptrie.html#update/4","title":"Iptrie.update/4","type":"function"},{"doc":"Returns all the values stored in all radix trees in trie . Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; values ( ipt ) [ 1 , 2 , 3 , 4 ]","ref":"Iptrie.html#values/1","title":"Iptrie.values/1","type":"function"},{"doc":"Returns the values stored in the radix trees in trie for given type . Where type is a either single maxlen or a list thereof. Example iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 1 ) ...&gt; |&gt; put ( &quot;2.2.2.0/24&quot; , 2 ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , 3 ) ...&gt; |&gt; put ( &quot;acdc:2021::/32&quot; , 4 ) iex&gt; iex&gt; values ( ipt , 32 ) [ 1 , 2 ] iex&gt; iex&gt; values ( ipt , 128 ) [ 3 , 4 ] iex&gt; iex&gt; values ( ipt , 48 ) [ ]","ref":"Iptrie.html#values/2","title":"Iptrie.values/2","type":"function"},{"doc":"A prefix represented as an Pfx.t/0 struct, an Pfx.ip_address/0 , Pfx.ip_prefix/0 or a string in IPv4 CIDR, IPv6, EUI-48 or EUI-64 format.","ref":"Iptrie.html#t:prefix/0","title":"Iptrie.prefix/0","type":"type"},{"doc":"An Iptrie struct that contains a Radix tree per type of Pfx.t/0 used. A prefix' type is determined by its maxlen property: IPv4 has maxlen: 32 , IPv6 has maxlen: 128 , MAC addresses have maxlen: 48 and so on. Although Iptrie facilitates (exact or longest prefix match) lookups of any type of prefix, it has a bias towards IP prefixes. So, any binaries (strings) are first interpreted as IPv4 CIDR/IPv6 strings and as EUI-48/64 string second, while tuples of address digits and/or {address-digits, length} are interpreted as IPv4 or IPv6 representations.","ref":"Iptrie.html#t:t/0","title":"Iptrie.t/0","type":"type"},{"doc":"The type of a prefix is its maxlen property.","ref":"Iptrie.html#t:type/0","title":"Iptrie.type/0","type":"type"},{"doc":"Functions to access a local snapshot of the IANA IPv4/6 Special-Purpose Address Registries. See also: IPv4 Special-Purpose Address Registry IPv6 Special-Purpose Address Registry At compile time, this module builds an Iptrie.t using Pfx's local copy of the IANA special-purpose address registries, which can then be accessed using: Iptrie.Iana.get/1 to retrieve the list of prefixes and properties for IPv4 or IPv6 prefixes Iptrie.Iana.lookup/2 to retrieve a single or all properties for a given prefix, and Iptrie.iana_special/2 , which delegates to Iptrie.Iana.lookup/2 . Running mix iana.specials will show the number of records and last update of these registries by Iana, as well as that of the local snapshot. If the snapshot is out-of-date, it will be updated with the new information, in which case both Pfx and Iptrie.Iana module will need to be recompiled. Examples iex&gt; Iptrie.Iana . lookup ( &quot;10.10.10.10&quot; ) { &quot;10.0.0.0/8&quot; , %{ allocation : &quot;1996-02&quot; , destination : true , forward : true , global : false , name : &quot;private-use&quot; , prefix : &quot;10.0.0.0/8&quot; , reserved : false , source : true , spec : [ &quot;rfc1918&quot; ] , termination : :na } } iex&gt; Iptrie . iana_special ( &quot;fc00::&quot; ) { &quot;fc00::/7&quot; , %{ allocation : &quot;2005-10&quot; , destination : true , forward : true , global : false , name : &quot;unique-local&quot; , prefix : &quot;fc00::/7&quot; , reserved : false , source : true , spec : [ &quot;rfc4193&quot; , &quot;rfc8190&quot; ] , termination : :na } } # retrieve a single property iex&gt; Iptrie.Iana . lookup ( &quot;fc00::&quot; , :global ) false iex&gt; Iptrie.Iana . lookup ( &quot;fc00::&quot; , :name ) &quot;unique-local&quot;","ref":"Iptrie.Iana.html","title":"Iptrie.Iana","type":"module"},{"doc":"Returns the list of IPv4 or IPv6 prefixes and their IANA special purpose address properties in a map. Examples iex&gt; get ( :ip4 ) |&gt; length ( ) 25 iex&gt; get ( :ip4 ) |&gt; hd ( ) { % Pfx { bits : &lt;&lt; 0 , 0 , 0 , 0 &gt;&gt; , maxlen : 32 } , %{ allocation : &quot;1981-09&quot; , destination : false , forward : false , global : false , name : &quot;this-host-on-this-network&quot; , prefix : &quot;0.0.0.0/32&quot; , reserved : true , source : true , spec : [ &quot;rfc1122&quot; ] , termination : :na } } iex&gt; get ( :ip6 ) |&gt; length ( ) 20 iex&gt; get ( :ip6 ) |&gt; hd ( ) { % Pfx { bits : &lt;&lt; 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 &gt;&gt; , maxlen : 128 } , %{ allocation : &quot;2006-02&quot; , destination : false , forward : false , global : false , name : &quot;unspecified-address&quot; , prefix : &quot;::/128&quot; , reserved : true , source : true , spec : [ &quot;rfc4291&quot; ] , termination : :na } } # get all non-globally-routed IPv4 prefixes iex&gt; get ( :ip4 ) ...&gt; |&gt; Enum . filter ( fn { _ , m } -&gt; m . global != true end ) ...&gt; |&gt; Enum . map ( fn { pfx , _ } -&gt; &quot; \#{ pfx } &quot; end ) [ &quot;0.0.0.0&quot; , &quot;0.0.0.0/8&quot; , &quot;10.0.0.0/8&quot; , &quot;100.64.0.0/10&quot; , &quot;127.0.0.0/8&quot; , &quot;169.254.0.0/16&quot; , &quot;172.16.0.0/12&quot; , &quot;192.0.0.0/29&quot; , &quot;192.0.0.0/24&quot; , &quot;192.0.0.8&quot; , &quot;192.0.0.170&quot; , &quot;192.0.0.171&quot; , &quot;192.0.2.0/24&quot; , &quot;192.88.99.0/24&quot; , &quot;192.168.0.0/16&quot; , &quot;198.18.0.0/15&quot; , &quot;198.51.100.0/24&quot; , &quot;203.0.113.0/24&quot; , &quot;240.0.0.0/4&quot; , &quot;255.255.255.255&quot; ]","ref":"Iptrie.Iana.html#get/1","title":"Iptrie.Iana.get/1","type":"function"},{"doc":"Performs a longest prefix match against the local IPv4 and IPv6 special purpose address registries and Returns either nil, a property value or property map for given search prefix. Examples iex(16)&gt; lookup ( &quot;0.0.0.0&quot; ) { &quot;0.0.0.0&quot; , %{ allocation : &quot;1981-09&quot; , destination : false , forward : false , global : false , name : &quot;this-host-on-this-network&quot; , prefix : &quot;0.0.0.0/32&quot; , reserved : true , source : true , spec : [ &quot;rfc1122&quot; ] , termination : :na } } iex(17)&gt; lookup ( &quot;0.0.0.1&quot; ) { &quot;0.0.0.0/8&quot; , %{ allocation : &quot;1981-09&quot; , destination : false , forward : false , global : false , name : &quot;this-network&quot; , prefix : &quot;0.0.0.0/8&quot; , reserved : true , source : true , spec : [ &quot;rfc791&quot; ] , termination : :na } } # a non-existing property `:missing` iex&gt; lookup ( &quot;10.10.10.10&quot; , :missing ) nil iex&gt; lookup ( &quot;::ffff:0:0/96&quot; , :global ) false iex&gt; lookup ( &quot;::ffff:0:0/96&quot; , :name ) &quot;ipv4-mapped-address&quot;","ref":"Iptrie.Iana.html#lookup/2","title":"Iptrie.Iana.lookup/2","type":"function"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"fixed some documentation added Iptrie.minimize/2 to easily minimize an Iptrie.","ref":"changelog.html#v0-9-0-2023-04-16","title":"Changelog - [v0.9.0] - 2023-04-16","type":"extras"},{"doc":"changed updated Pfx to latest version, so IPv6 is formatted with shorthand notation tests were modified accordingly added Iptrie.iana_special/2 which delegates to Iptrie.Iana.lookup/2","ref":"changelog.html#v0-8-0-2022-02-20","title":"Changelog - [v0.8.0] - 2022-02-20","type":"extras"},{"doc":"added Iptrie.Iana module to access IANA IPv4/6 Specical-Purpose Address Registries changed updated dependency Pfx to v0.12.0 fixed typespec for Iptrie.t/0 by adding optional integer -&gt; Radix.t","ref":"changelog.html#v0-7-0-2022-01-16","title":"Changelog - [v0.7.0] - 2022-01-16","type":"extras"},{"doc":"added Iptrie.get_and_update/3 to do update a key,value-pair in one go. changed Iptrie.less/3 now can optionally exclude search key from results. Iptrie.more/3 now can optionally exclude search key from results. fixed Readme examples write their files to assets/, not img/","ref":"changelog.html#v0-6-0-2021-12-04","title":"Changelog - [v0.6.0] - 2021-12-04","type":"extras"},{"doc":"changed Pulled in Radix 0.3.0, for Radix.prune/3 added Iptrie.prune/3 to prune an Iptrie, optionally doing so recursively","ref":"changelog.html#v0-5-0-2021-07-25","title":"Changelog - v0.5.0 - 2021-07-25","type":"extras"},{"doc":"","ref":"changelog.html#v0-4-0-2021-07-20","title":"Changelog - v0.4.0 - 2021-07-20","type":"extras"},{"doc":"Merged #1 misc doc changes Pulled in Pfx 0.5.0 Pulled in Radix 0.2.0 Updated deps with new versions of Pfx and Radix Iptrie.delete/2 only deletes a single prefix, now Iptrie.drop/2 is available Iptrie.filter/2 user callback now takes a Pfx.t/0 instead of bitstring radix key Iptrie.get/2 no longer gets a list of prefixes, just 1 prefix at a time Iptrie.keys/2 returns keys for a single type only Iptrie.reduce/3 user callback now takes a Pfx.t/0 instead of bitstring radix key Iptrie.reduce/4 takes a single type only, not a list of types Iptrie.reduce/4 user callback now takes a Pfx.t/0 instead of bitstring radix key Iptrie.to_list/2 returns prefix,value-pairs for a single type only Iptrie.values/2 returns values for a single radix tree type only","ref":"changelog.html#changed","title":"Changelog - changed","type":"extras"},{"doc":"Iptrie.count/1 , return count of all entries in an Iptrie Iptrie.count/2 , return count of entries for given type Iptrie.drop/2 , drop some prefixes from the Iptrie Iptrie.empty?/1 , says if a given Iptrie is empty or not Iptrie.empty?/2 , says if a particular radix tree of type is empty or not Iptrie.get/3 , gets a prefix or returns default if prefix was not found Iptrie.has_prefix?/2 , says whether given prefix is present in an Iptrie Iptrie.has_type?/2 , says whether or not trie has a given type Iptrie.merge/2 , merges two Iptrie's Iptrie.merge/3 , merges two Iptrie's with conflict resolution Iptrie.pop/3 , returns popped pfx,val and new trie Iptrie.radix/2 , get a radix tree by type from an Iptrie, or a new empty one Iptrie.split/3 , split one trie into two (optionally using lpm) Iptrie.take/3 , return new Iptrie contains just the given keys Iptrie.types/1 , returns a list of available types or maxlen's in the trie","ref":"changelog.html#added","title":"Changelog - added","type":"extras"},{"doc":"added Iptrie.fetch/2 , Iptrie.fetch!/2 Iptrie.find/2 , Iptrie.find!/2 Iptrie.filter/2","ref":"changelog.html#v0-3-0-2021-07-06","title":"Changelog - v0.3.0 - 2021-07-06","type":"extras"},{"doc":"added Iptrie.radix/2 Iptrie.reduce/3 , Iptrie.reduce/4 Iptrie.to_list/1 , Iptrie.to_list/2","ref":"changelog.html#v0-2-0-2021-07-05","title":"Changelog - v0.2.0 - 2021-07-05","type":"extras"},{"doc":"Initial public version","ref":"changelog.html#v0-1-0-2021-07-05","title":"Changelog - v0.1.0 - 2021-07-05","type":"extras"},{"doc":"Copyright (c) 2021 hertogp Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"IP lookup, with longest prefix match, for IPv4, IPv6 prefixes (and others). Iptrie manages multiple Radix trees, one for each type of prefix used as determined by their maxlen property after encoding it as a Pfx.t/0 struct. That way, IPv4 prefixes ( maxlen: 32 ) use a different radix tree as opposed to IPv6 ( maxlen: 128 ) or other types. Iptrie has a bias towards IPv4, IPv6, EUI-48 and EUI-64, since it uses Pfx to convert arguments to a Pfx.t/0 struct. Other types of prefixes will require the actual Pfx.t/0 structs as arguments for the various Iptrie functions. Like Pfx , Iptrie tries to mirror the representation of results to the arguments given, if possible.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;1.2.3.0/24&quot; , &quot;v4&quot; ) ...&gt; |&gt; put ( &quot;128.0.0.0/8&quot; , &quot;v4-128&quot; ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; ) ...&gt; |&gt; put ( &quot;acdc:1978::/32&quot; , &quot;Powerage&quot; ) ...&gt; |&gt; put ( &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; ) ...&gt; |&gt; put ( &quot;::/0&quot; , &quot;no dynamite&quot; ) iex&gt; iex&gt; # 3 longest prefix matches that find the same prefix iex&gt; lookup ( ipt , &quot;1.2.3.128&quot; ) { &quot;1.2.3.0/24&quot; , &quot;v4&quot; } iex&gt; lookup ( ipt , { { 1 , 2 , 3 , 128 } , 32 } ) { { { 1 , 2 , 3 , 0 } , 24 } , &quot;v4&quot; } iex&gt; iex&gt; lookup ( ipt , % Pfx { bits : &lt;&lt; 1 , 2 , 3 , 128 &gt;&gt; , maxlen : 32 } ) { % Pfx { bits : &lt;&lt; 1 , 2 , 3 &gt;&gt; , maxlen : 32 } , &quot;v4&quot; } iex&gt; iex&gt; lookup ( ipt , &quot;acdc:1975::&quot; ) { &quot;acdc:1975::/32&quot; , &quot;T.N.T&quot; } iex&gt; iex&gt; # separate trees, separate default &quot;routes&quot; iex&gt; lookup ( ipt , &quot;10.11.12.13&quot; ) { &quot;0.0.0.0/0&quot; , &quot;v4 default&quot; } iex&gt; lookup ( ipt , &quot;abba::&quot; ) { &quot;::/0&quot; , &quot;no dynamite&quot; } iex&gt; iex&gt; # visualize the IPv4 &amp; IPv6 radix trees iex&gt; kv32 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 32 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 32 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv4&quot; , kv_tostr : kv32 ) ...&gt; |&gt; ( &amp; File . write ( &quot;assets/ipv4.dot&quot; , &amp;1 ) ) . ( ) iex&gt; kv128 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 128 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 128 ) ...&gt; |&gt; Radix . dot ( label : &quot;IPv6&quot; , kv_tostr : kv128 ) ...&gt; |&gt; ( &amp; File . write ( &quot;assets/ipv6.dot&quot; , &amp;1 ) ) . ( ) The radix trees in the example above look like this:","ref":"readme.html#ipv4-ipv6","title":"Overview - IPv4/IPv6","type":"extras"},{"doc":"Iptrie can also be used to do longest prefix match lookup for other types of prefixes, like e.g. MAC addresses: iex&gt; ipt = new ( ) ...&gt; |&gt; put ( &quot;00-22-72-00-00-00/24&quot; , &quot;American Micro-Fuel Device&quot; ) ...&gt; |&gt; put ( &quot;00-d0-ef-00-00-00/24&quot; , &quot;IGT&quot; ) ...&gt; |&gt; put ( &quot;08-61-95-00-00-00/24&quot; , &quot;Rockwell Automation&quot; ) iex&gt; iex&gt; lookup ( ipt , &quot;00-d0-ef-aa-bb-cc&quot; ) { &quot;00-D0-EF-00-00-00/24&quot; , &quot;IGT&quot; } iex&gt; iex&gt; # longest match for partial prefix iex&gt; lookup ( ipt , &quot;08-61-95-11-22-00/40&quot; ) |&gt; elem ( 1 ) &quot;Rockwell Automation&quot; iex&gt; iex&gt; kv48 = fn { k , v } -&gt; &quot; \#{ Pfx . new ( k , 48 ) } &lt;br/&gt; \#{ v } &quot; end iex&gt; radix ( ipt , 48 ) ...&gt; |&gt; Radix . dot ( label : &quot;MAC OUI&quot; , kv_tostr : kv48 ) ...&gt; |&gt; ( &amp; File . write ( &quot;assets/mac.dot&quot; , &amp;1 ) ) . ( ) Iptrie recognizes EUI-48 and EUI-64 prefixes. Note that EUI-64 using ':' as punctuation might come out as an IPv6, in which case Pfx.from_mac/1 should be used to create a prefix before putting it in the trie. Since prefixes are stored in specific radix trees based on the maxlen of given prefix, you could also mix IPv4, IPv6, EUI-48, EUI-64 prefixes and possibly others, in a single Iptrie.","ref":"readme.html#others","title":"Overview - Others","type":"extras"},{"doc":"The Iptrie.Iana module enables retrieval of properties for prefixes registered in one of the IPv4 or IPv6 special-purpose address registries. iex&gt; alias Iptrie.Iana iex&gt; Iana . lookup ( &quot;10.11.12.13&quot; ) { &quot;10.0.0.0/8&quot; , %{ allocation : &quot;1996-02&quot; , destination : true , forward : true , global : false , name : &quot;private-use&quot; , prefix : &quot;10.0.0.0/8&quot; , reserved : false , source : true , spec : [ &quot;rfc1918&quot; ] , termination : :na } } # or access some property directly iex&gt; alias Iptrie.Iana iex&gt; Iana . lookup ( &quot;2001::&quot; , :name ) &quot;teredo&quot;","ref":"readme.html#iana-special-purpose-registries","title":"Overview - IANA Special-Purpose Registries","type":"extras"},{"doc":"Minimize an Iptrie to its bare minimum by combining neighbouring prefixes and removing more specific prefixes. iex&gt; f = fn v1 , v2 -&gt; if v1 == v2 , do : { :ok , v1 } , else : nil end iex&gt; [ ...&gt; { &quot;1.1.1.0/25&quot; , true } , ...&gt; { &quot;1.1.1.128/25&quot; , true } , ...&gt; { &quot;2.2.2.0/24&quot; , true } , ...&gt; { &quot;2.2.2.0/28&quot; , true } , ...&gt; { &quot;2.2.2.2&quot; , true } ...&gt; ] ...&gt; |&gt; new ( ) ...&gt; |&gt; minimize ( f ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { k , v } -&gt; { &quot; \#{ k } &quot; , v } end ) [ { &quot;1.1.1.0/24&quot; , true } , { &quot;2.2.2.0/24&quot; , true } ] Summarize an Iptrie containing a list of full length prefixes within a /24, with 1 missing address: iex&gt; new ( for x &lt;- 0 .. 255 , do : { &quot;1.1.1. \#{ x } &quot; , x } ) ...&gt; |&gt; delete ( &quot;1.1.1.128&quot; ) ...&gt; |&gt; prune ( fn _ -&gt; { :ok , 0 } end , recurse : true ) ...&gt; |&gt; to_list ( ) ...&gt; |&gt; Enum . map ( fn { pfx , _v } -&gt; { &quot; \#{ pfx } &quot; , &quot;Range \#{ Pfx . first ( pfx ) } - \#{ Pfx . last ( pfx ) } &quot; } end ) [ { &quot;1.1.1.0/25&quot; , &quot;Range 1.1.1.0 - 1.1.1.127&quot; } , { &quot;1.1.1.129&quot; , &quot;Range 1.1.1.129 - 1.1.1.129&quot; } , { &quot;1.1.1.130/31&quot; , &quot;Range 1.1.1.130 - 1.1.1.131&quot; } , { &quot;1.1.1.132/30&quot; , &quot;Range 1.1.1.132 - 1.1.1.135&quot; } , { &quot;1.1.1.136/29&quot; , &quot;Range 1.1.1.136 - 1.1.1.143&quot; } , { &quot;1.1.1.144/28&quot; , &quot;Range 1.1.1.144 - 1.1.1.159&quot; } , { &quot;1.1.1.160/27&quot; , &quot;Range 1.1.1.160 - 1.1.1.191&quot; } , { &quot;1.1.1.192/26&quot; , &quot;Range 1.1.1.192 - 1.1.1.255&quot; } ] Find the more specific entries in an Iptrie for a given search prefix iex&gt; new ( ) ...&gt; |&gt; put ( &quot;acdc:1975::/32&quot; , &quot;TNT&quot; ) ...&gt; |&gt; put ( &quot;acdc:1977::/32&quot; , &quot;LTBR&quot; ) ...&gt; |&gt; put ( &quot;abba:1975::/32&quot; , &quot;RING&quot; ) ...&gt; |&gt; more ( &quot;acdc::/16&quot; ) ...&gt; |&gt; Enum . map ( fn { p , v } -&gt; { &quot; \#{ p } &quot; , v } end ) [ { &quot;acdc:1977::/32&quot; , &quot;LTBR&quot; } , { &quot;acdc:1975::/32&quot; , &quot;TNT&quot; } ] Find the less specific entries in an Iptrie for a given search prefix iex&gt; new ( ) ...&gt; |&gt; put ( &quot;1.1.1.0/24&quot; , 0 ) ...&gt; |&gt; put ( &quot;1.1.1.0/25&quot; , 1 ) ...&gt; |&gt; put ( &quot;1.1.1.0/26&quot; , 2 ) ...&gt; |&gt; put ( &quot;1.1.1.0/30&quot; , 3 ) ...&gt; |&gt; put ( &quot;1.1.1.128/25&quot; , 4 ) ...&gt; |&gt; less ( &quot;1.1.1.3&quot; ) ...&gt; |&gt; Enum . map ( fn { pfx , v } -&gt; { &quot; \#{ pfx } &quot; , v } end ) [ { &quot;1.1.1.0/30&quot; , 3 } , { &quot;1.1.1.0/26&quot; , 2 } , { &quot;1.1.1.0/25&quot; , 1 } , { &quot;1.1.1.0/24&quot; , 0 } ]","ref":"readme.html#examples","title":"Overview - Examples","type":"extras"},{"doc":"Iptrie can be installed by adding :iptrie to your list of dependencies in mix.exs : def deps do [ { :iptrie , &quot;~&gt; 0.9&quot; } ] end","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"Copyright (c) 2021 hertogp The source code is licensed under the MIT License .","ref":"readme.html#copyright-and-license","title":"Overview - Copyright and License","type":"extras"}]